//*****************************************************************************
// Description     : Main sample program to measure histogram
// Target Device   : APU101
// Target Platform : Linux, C
// Date            : July 2017
// Version         : 1.0
//*****************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>

#include "file.h"
#include "sitcp.h"
#include "device.h"
#include "local_define.h"


//
// Local functions declaration
//
static int  config_device(DEVICE_COM_INFO*, unsigned qword);
static int  run_measurement(DEVICE_COM_INFO*, unsigned qword);
static int  receive_data(DEVICE_COM_INFO*);


//#########################################################################
//  Name      : main
//  Function  : entry point
//  Arguments :
//      in argc : number of parameters
//      in argv : parameters
//  Return :
//      0 : success
//      1 : failure
//  Note : 
//#########################################################################
int main(int argc, char **argv)
{
    int run_ready;
    int ret;
    unsigned qword  meas_count;
    DEVICE_COM_INFO cominfo;

    run_ready = 0;
    memset(&cominfo, 0, sizeof(cominfo));

    // 1. connect to the device
    ret = device_connect(&cominfo);
    if (ret != 0) {
        printf("device connect error(%d)...\n", ret);
        return(1);
    }

    // 2. set parameters to the device
    meas_count = device_msec2count(MEASUREMENT_TIME);   // convert msec to count
    ret = config_device(&cominfo, meas_count);
    if (ret == 0) {
        run_ready = 1;
    }
    else {
        printf("setting device config error(%d)...\n", ret);
    }

    if (run_ready == 1) {   // ready to run
        // 3. start measurement
        printf("start measurement!!!\n");
        ret = run_measurement(&cominfo, meas_count);

        printf("finish measurement...\n");

        if (ret >= 0) {
            printf("measurement completed!!!\n");
        }
        else {
            printf("measurement error(%d)...\n", ret);
        }
    }

    // 4. post processing
    device_disconnect(&cominfo);

    return(0);
}


//#########################################################################
//  Name      : config_device
//  Function  : set parameters to device
//  Arguments :
//      in pComInfo : pointer to the structure data for referring sockets
//      in ms_count : measurement time (counts)
//  Return :
//       0 : success
//      -n : failed at nth parameter
//  Note : 
//#########################################################################
static int config_device(DEVICE_COM_INFO *pComInfo, unsigned qword ms_count)
{
    int ret;

    // polarity: positive
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_POL, DEVICE_PARAM_POL_POS);
    if (ret < 0)  { return(-1); }

    // analog coarse gain: x5
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_ACG, DEVICE_PARAM_ACG_x5);
    if (ret < 0)  { return(-2); }

    // ADC gain: 4096
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_ADG, DEVICE_PARAM_ADG_4096);
    if (ret < 0)  { return(-3); }

    // fast trigger threshold: 30
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_FTH, 30);
    if (ret < 0)  { return(-4); }

    // fast filter pole zero: 0
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_FPZ, 0);
    if (ret < 0)  { return(-5); }

    // slow filter rise time: 1000ns(100 @1digit=10ns)
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_SFR, 100);
    if (ret < 0)  { return(-6); }

    // slow filter peaking time: 1100ns(110 @1digit=10ns)
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_SFP, 110);
    if (ret < 0)  { return(-7); }

    // slow filter pole zero: 660ns(66 @1digit=10ns)
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_SPZ, 66);
    if (ret < 0)  { return(-8); }

    // slow filter threshold: 40
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_STH, 40);
    if (ret < 0)  { return(-9); }

    // digital coarse gain: x16
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_DCG, DEVICE_PARAM_DCG_x16);
    if (ret < 0)  { return(-10); }

    // digital fine gain: x0.5 (8193 x 0.5 - 2 = 4094)
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_DFG, 4094);
    if (ret < 0)  { return(-11); }

    // reset filter: 0,1,0
    ret = device_write_3step(pComInfo, DEVICE_COMMAND_RFL);
    if (ret < 0)  { return(-12); }

    // DAC monitor: slow
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_DAC, DEVICE_PARAM_DAC_SLOW);
    if (ret < 0)  { return(-13); }

    // measurement mode: histogram
    ret = device_write_16bit(pComInfo, DEVICE_COMMAND_MOD, DEVICE_PARAM_MOD_HIST);
    if (ret < 0)  { return(-14); }

    // measurement time
    ret = device_write_48bit(pComInfo, DEVICE_COMMAND_MTM, ms_count);
    if (ret < 0)  { return(-15); }

    return(0);
}


//#########################################################################
//  Name      : run_measurement
//  Function  : measurement processing
//  Arguments :
//      in pComInfo : pointer to the structure data for referring sockets
//      in ms_count : measurement time (counts)
//  Return :
//       0 : success
//      -1 : failed while measurement
//      -2 : failed to write file header
//      -3 : failed to receive histogram data
//  Note : 
//#########################################################################
static int run_measurement(DEVICE_COM_INFO *pComInfo, unsigned qword ms_count)
{
    int ret;
    unsigned qword now_count;
    unsigned int now_msec;
    unsigned short status;
    unsigned int input_cps;
    unsigned int throughput_cps;

    // clear data
    device_write_3step(pComInfo, DEVICE_COMMAND_CLR);
    // wait for completion of clear processing
    msleep(100);

    // start measurement
    device_start_measurement(pComInfo);

    while(1) {
        // read status
        ret = device_read_status(pComInfo, &status, &now_count, &input_cps, &throughput_cps);

        if (ret < 0) { // communication error
            // quit measurement
            break;
        }
        // status is 'stopped' or measurement time elapsed
        if ((status == DEVICE_PARAM_AQS_STOP) || (now_count >= ms_count)) {
            break;
        }

        msleep(200);   // wait for 200ms
    }

    // stop measurement
    device_stop_measurement(pComInfo);

    // quit by error
    if (ret < 0) {
        return(-1);
    }

    // write file header
    now_msec  = (unsigned int)(device_count2msec(now_count));  // convert count to msec
    ret =  file_write_header(SAVE_FILENAME, now_msec, input_cps, throughput_cps);
    if (ret < 0) {
        return(-2);
    }

    // receive histogram data then write out to file
    ret = receive_data(pComInfo);
    if (ret < 0) {
        return(-3);
    } else {
        return(0);
    }
}


//#########################################################################
//  Name      : receive_data
//  Function  : receiving collected data
//  Arguments :
//      in pComInfo : pointer to the structure data for referring sockets
//  Return :
//       0 : success
//      -1 : failed to receive
//      -2 : failed to write file
//  Note : 
//#########################################################################
static int receive_data(DEVICE_COM_INFO *pComInfo)
{
    int ret;
    int result;
    unsigned int hdata[8192];  // for APU101
//  unsigned int hdata[4096];  // for APU101X

    result = 0;

    // receiving histogram data
    memset(hdata, 0, sizeof(hdata));
    ret = device_read_histogram(pComInfo,
                               (char*)(hdata), sizeof(hdata),
                                DEVICE_TIMEOUT_DATA);

    if (ret > 0) {
        // write data to file
        ret =  file_write_data(SAVE_FILENAME, sizeof(hdata), (void *)hdata);
        if (ret < 0) {
            // failed to write
            result = -2;
        }
    } else {
        // failed to receive data
        result = -1;
    }

    return(result);
}
